"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("../helpers");
class ExceptionManager {
    constructor(container) {
        this._resolver = container.getResolver();
    }
    registerHandler(handler) {
        if (typeof (handler) === 'string') {
            this._resolvedErrorHandler = this._resolver.resolve(handler);
        }
        else {
            this._resolvedErrorHandler = {
                type: 'function',
                value: handler,
            };
        }
    }
    async handle(error, ctx) {
        if (!this._resolvedErrorHandler) {
            ctx.response.status(error.status || 500).send(error.message);
            return;
        }
        try {
            let value = null;
            if (this._resolvedErrorHandler.type === 'function') {
                value = await this._resolvedErrorHandler.value(error, ctx);
            }
            else {
                value = await this._resolver.call(this._resolvedErrorHandler, undefined, [error, ctx]);
            }
            if (helpers_1.useReturnValue(value, ctx)) {
                ctx.response.send(value);
            }
        }
        catch (finalError) {
            ctx.response.status(error.status || 500).send(error.message);
            ctx.logger.fatal(finalError, 'Unexpected exception raised from http exception handler');
        }
    }
}
exports.ExceptionManager = ExceptionManager;
