"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const haye_1 = __importDefault(require("haye"));
const co_compose_1 = require("co-compose");
const utils_1 = require("@poppinss/utils");
const helpers_1 = require("../../helpers");
class PreCompiler {
    constructor(container, _middlewareStore) {
        this._middlewareStore = _middlewareStore;
        this._finalRouteHandler = async function finalRouteHandler(ctx) {
            let data = {};
            const controllerRow = ctx.profiler.child('http:route:handler', data);
            ctx.profiler = controllerRow;
            const routeHandler = ctx.route.meta.resolvedHandler;
            let returnValue;
            try {
                if (routeHandler.type === 'function') {
                    returnValue = await routeHandler.handler(ctx);
                }
                else {
                    data.controller = routeHandler.namespace;
                    data.method = routeHandler.method;
                    returnValue = await this._resolver.call(routeHandler, ctx.route.meta.namespace, [ctx]);
                }
                if (helpers_1.useReturnValue(returnValue, ctx)) {
                    ctx.response.send(returnValue);
                }
                controllerRow.end();
            }
            catch (error) {
                controllerRow.end();
                throw error;
            }
        }.bind(this);
        this._routeMiddlewareHandler = async function routeMiddlewareHandler(ctx) {
            await new co_compose_1.Middleware()
                .register(ctx.route.meta.resolvedMiddleware)
                .runner()
                .resolve(this._middlewareStore.invokeMiddleware.bind(this._middlewareStore))
                .finalHandler(this._finalRouteHandler, [ctx])
                .run([ctx]);
        }.bind(this);
        this._resolver = container.getResolver(undefined, 'httpControllers', 'App/Controllers/Http');
    }
    _compileHandler(route) {
        if (typeof (route.handler) === 'string') {
            route.meta.resolvedHandler = this._resolver.resolve(route.handler, route.meta.namespace);
        }
        else {
            route.meta.resolvedHandler = { type: 'function', handler: route.handler };
        }
    }
    _compileMiddleware(route) {
        route.meta.resolvedMiddleware = route.middleware.map((item) => {
            if (typeof (item) === 'function') {
                return { type: 'function', value: item, args: [] };
            }
            const [{ name, args }] = haye_1.default.fromPipe(item).toArray();
            const resolvedMiddleware = this._middlewareStore.getNamed(name);
            if (!resolvedMiddleware) {
                throw new utils_1.Exception(`Cannot find named middleware ${name}`, 500, 'E_MISSING_NAMED_MIDDLEWARE');
            }
            resolvedMiddleware.args = args;
            return resolvedMiddleware;
        });
    }
    _setFinalHandler(route) {
        if (route.meta.resolvedMiddleware && route.meta.resolvedMiddleware.length) {
            route.meta.finalHandler = this._routeMiddlewareHandler;
        }
        else {
            route.meta.finalHandler = this._finalRouteHandler;
        }
    }
    compileRoute(route) {
        this._compileHandler(route);
        this._compileMiddleware(route);
        this._setFinalHandler(route);
    }
}
exports.PreCompiler = PreCompiler;
